@@ -36,10 +36,12 @@ from awscli.customizations.s3.fileinfobuilder import FileInfoBuilder
 from awscli.customizations.s3.filters import create_filter
 from awscli.customizations.s3.s3handler import S3TransferHandlerFactory
 from awscli.customizations.s3.syncstrategy.base import (
+    AlwaysSync,
     MissingFileSync,
     NeverSync,
     SizeAndLastModifiedSync,
 )
+from awscli.customizations.s3.syncstrategy.caseconflict import CaseConflictSync
 from awscli.customizations.s3.utils import (
     AppendFilter,
     RequestParamsMapper,
@@ -543,9 +545,7 @@ PROGRESS_MULTILINE = {
     'name': 'progress-multiline',
     'dest': 'progress_multiline',
     'action': 'store_true',
-    'help_text': (
-        'Show progress on multiple lines.'
-    ),
+    'help_text': ('Show progress on multiple lines.'),
 }
 
 
@@ -664,6 +664,44 @@ BUCKET_REGION = {
     ),
 }
 
+NO_OVERWRITE = {
+    'name': 'no-overwrite',
+    'action': 'store_true',
+    'help_text': (
+        "This flag prevents overwriting of files at the destination. With this flag, "
+        "only files not present at the destination will be transferred."
+    ),
+}
+
+
+CASE_CONFLICT = {
+    'name': 'case-conflict',
+    'choices': [
+        'ignore',
+        'skip',
+        'warn',
+        'error',
+    ],
+    'default': 'ignore',
+    'help_text': (
+        "Configures behavior when attempting to download multiple objects "
+        "whose keys differ only by case, which can cause undefined behavior "
+        "on case-insensitive filesystems. "
+        "This parameter only applies for commands that perform multiple S3 "
+        "to local downloads. "
+        f"See <a href='{CaseConflictSync.DOC_URI}'>Handling case "
+        "conflicts</a> for details. Valid values are: "
+        "<ul>"
+        "<li>``error`` - Raise an error and abort downloads.</li>"
+        "<li>``warn`` - Emit a warning and download the object.</li>"
+        "<li>``skip`` - Skip downloading the object.</li>"
+        "<li>``ignore`` - The default value. Ignore the conflict and "
+        "download the object.</li>"
+        "</ul>"
+    ),
+}
+
+
 TRANSFER_ARGS = [
     DRYRUN,
     QUIET,
@@ -1081,7 +1119,15 @@ class CpCommand(S3TransferCommand):
             }
         ]
         + TRANSFER_ARGS
-        + [METADATA, COPY_PROPS, METADATA_DIRECTIVE, EXPECTED_SIZE, RECURSIVE]
+        + [
+            METADATA,
+            COPY_PROPS,
+            METADATA_DIRECTIVE,
+            EXPECTED_SIZE,
+            RECURSIVE,
+            CASE_CONFLICT,
+            NO_OVERWRITE,
+        ]
     )
 
 
@@ -1105,6 +1151,8 @@ class MvCommand(S3TransferCommand):
             METADATA_DIRECTIVE,
             RECURSIVE,
             VALIDATE_SAME_S3_PATHS,
+            CASE_CONFLICT,
+            NO_OVERWRITE,
         ]
     )
 
@@ -1150,7 +1198,7 @@ class SyncCommand(S3TransferCommand):
             }
         ]
         + TRANSFER_ARGS
-        + [METADATA, COPY_PROPS, METADATA_DIRECTIVE]
+        + [METADATA, COPY_PROPS, METADATA_DIRECTIVE, CASE_CONFLICT, NO_OVERWRITE]
     )
 
 
@@ -1307,7 +1355,16 @@ class CommandArchitecture:
         sync_strategies['file_at_src_and_dest_sync_strategy'] = (
             SizeAndLastModifiedSync()
         )
-        sync_strategies['file_not_at_dest_sync_strategy'] = MissingFileSync()
+        if self._should_handle_case_conflicts():
+            sync_strategies['file_not_at_dest_sync_strategy'] = (
+                CaseConflictSync(
+                    on_case_conflict=self.parameters['case_conflict']
+                )
+            )
+        else:
+            sync_strategies['file_not_at_dest_sync_strategy'] = (
+                MissingFileSync()
+            )
         sync_strategies['file_not_at_src_sync_strategy'] = NeverSync()
 
         # Determine what strategies to override if any.
@@ -1321,7 +1378,6 @@ class CommandArchitecture:
                     sync_type = override_sync_strategy.sync_type
                     sync_type += '_sync_strategy'
                     sync_strategies[sync_type] = override_sync_strategy
-
         return sync_strategies
 
     def run(self):
@@ -1408,7 +1464,8 @@ class CommandArchitecture:
             self._client, self._source_client, self.parameters
         )
 
-        s3_transfer_handler = S3TransferHandlerFactory(self.parameters)(
+        params = self._get_s3_handler_params()
+        s3_transfer_handler = S3TransferHandlerFactory(params)(
             self._transfer_manager, result_queue
         )
 
@@ -1440,6 +1497,12 @@ class CommandArchitecture:
                 'file_info_builder': [file_info_builder],
                 's3_handler': [s3_transfer_handler],
             }
+            if self._should_handle_case_conflicts():
+                self._handle_case_conflicts(
+                    command_dict,
+                    rev_files,
+                    rev_generator,
+                )
         elif self.cmd == 'rm':
             command_dict = {
                 'setup': [files],
@@ -1456,6 +1519,12 @@ class CommandArchitecture:
                 'file_info_builder': [file_info_builder],
                 's3_handler': [s3_transfer_handler],
             }
+            if self._should_handle_case_conflicts():
+                self._handle_case_conflicts(
+                    command_dict,
+                    rev_files,
+                    rev_generator,
+                )
 
         files = command_dict['setup']
         while self.instructions:
@@ -1517,6 +1586,82 @@ class CommandArchitecture:
                 },
             )
 
+    def _get_s3_handler_params(self):
+        params = self.parameters.copy()
+        
+        # Removing no-overwrite params from sync since file to be synced are
+        # already separated out using sync strategy
+        if self.cmd == 'sync':
+            params.pop('no_overwrite', None)
+        return params
+    def _should_handle_case_conflicts(self):
+        return (
+            self.cmd in {'sync', 'cp', 'mv'}
+            and self.parameters.get('paths_type') == 's3local'
+            and self.parameters['case_conflict'] != 'ignore'
+            and self.parameters.get('dir_op')
+        )
+
+    def _handle_case_conflicts(self, command_dict, rev_files, rev_generator):
+        # Objects are not returned in lexicographical order when
+        # operated on S3 Express directory buckets. This is required
+        # for sync operations to behave correctly, which is what
+        # recursive copies and moves fall back to so potential case
+        # conflicts can be detected and handled.
+        if not is_s3express_bucket(
+            split_s3_bucket_key(self.parameters['src'])[0]
+        ):
+            self._modify_instructions_for_case_conflicts(
+                command_dict, rev_files, rev_generator
+            )
+            return
+        # `skip` and `error` are not valid choices in this case because
+        # it's not possible to detect case conflicts.
+        if self.parameters['case_conflict'] not in {'ignore', 'warn'}:
+            raise ParamValidationError(
+                f"`{self.parameters['case_conflict']}` is not a valid value "
+                "for `--case-conflict` when operating on S3 Express "
+                "directory buckets. Valid values: `warn`, `ignore`."
+            )
+        msg = (
+            "warning: Recursive copies/moves from an S3 Express "
+            "directory bucket to a case-insensitive local filesystem "
+            "may result in undefined behavior if there are "
+            "S3 object key names that differ only by case. To disable "
+            "this warning, set the `--case-conflict` parameter to `ignore`. "
+            f"For more information, see {CaseConflictSync.DOC_URI}."
+        )
+        uni_print(msg, sys.stderr)
+
+    def _modify_instructions_for_case_conflicts(
+        self, command_dict, rev_files, rev_generator
+    ):
+        # Command will perform recursive S3 to local downloads.
+        # Checking for potential case conflicts requires knowledge
+        # of local files. Instead of writing a separate validation
+        # mechanism for recursive downloads, we modify the instructions
+        # to mimic a sync command.
+        sync_strategies = {
+            # Local filename exists with exact case match. Always sync
+            # because it's a copy operation.
+            'file_at_src_and_dest_sync_strategy': AlwaysSync(),
+            # Local filename either doesn't exist or differs only by case.
+            # Let `CaseConflictSync` determine which it is and handle it
+            # according to configured `--case-conflict` parameter.
+            'file_not_at_dest_sync_strategy': CaseConflictSync(
+                on_case_conflict=self.parameters['case_conflict']
+            ),
+            # Copy is one-way so never sync if not at source.
+            'file_not_at_src_sync_strategy': NeverSync(),
+        }
+        command_dict['setup'].append(rev_files)
+        command_dict['file_generator'].append(rev_generator)
+        command_dict['filters'].append(create_filter(self.parameters))
+        command_dict['comparator'] = [Comparator(**sync_strategies)]
+        self.instructions.insert(
+            self.instructions.index('file_info_builder'), 'comparator'
+        )
+
 
 # TODO: This class is fairly quirky in the sense that it is both a builder
 #  and a data object. In the future we should make the following refactorings
@@ -1580,6 +1725,7 @@ class CommandParameters:
         elif len(paths) == 1:
             self.parameters['dest'] = paths[0]
         self._validate_streaming_paths()
+        self._validate_no_overwrite_for_download_streaming()
         self._validate_path_args()
         self._validate_sse_c_args()
         self._validate_not_s3_express_bucket_for_sync()
@@ -1831,3 +1977,14 @@ class CommandParameters:
                     '--sse-c-copy-source is only supported for '
                     'copy operations.'
                 )
+
+    def _validate_no_overwrite_for_download_streaming(self):
+        if (
+            self.parameters['is_stream']
+            and self.parameters.get('no_overwrite')
+            and self.parameters['dest'] == '-'
+        ):
+            raise ParamValidationError(
+                "--no-overwrite parameter is not supported for "
+                "streaming downloads"
+            )
